 /*
 * FILE								: hardware.c
 * DESCRIPTION				: This file is iHMI43 W25Q16/64 Driver.
 * Author							: XiaomaGee@Gmail.com
 * Copyright					:
 *
 * History
 * -------------------
 * Rev								: 0.00
 * Date								: 08/26/2012
 *
 * create.
 * -------------------
 */
//-----------------Include files-------------------------//
#include "..\include\spi.h"
#include "..\include\stm32f10x_reg.h"
#include "..\include\flash.h"


//--------------------define------------------------------//



//-----------------Function Prototype--------------------//

unsigned long read_flash_id(void);
void enable_flash_write(void);
void disable_flash_write(void);
void erase_flash(void);
void enable_write_status_register(void);
void write_status_register(unsigned char );
unsigned char read_status_register(void);
void read_cont(unsigned long /* addr */, unsigned long /* byte_num */);

void wait_busy(void);
void sector_erase(unsigned long);
void page_write(unsigned long int addr,char *buf);
int flash_writen(unsigned long int addr,char *buf, int len);
int flash_readn(unsigned long int addr,char *buf, int len);
//-----------------Variable-----------------------------//  
W25QXX_T w25qxx = {
	.read_id = read_flash_id,
	.writen = flash_writen,
	.readn = flash_readn
};
/*
 * Name								: page_write
 * Description				: ---
 * Author							: XiaomaGee.
 *
 * History
 * -------------------
 * Rev								: 0.00
 * Date								: 08/26/2012
 * 
 * create.
 * -------------------
 */

void page_write(unsigned long int addr, char *buf)
{
	int i;
	
	enable_flash_write();
	
	SPI1_CS_OFF;
													
	spi1.send_data(0x02); 				
	spi1.send_data(((addr  & 0xFFFFFF) >> 16)); 	/* send 3 address bytes */
	spi1.send_data(((addr  & 0xFFFF) >> 8));
	spi1.send_data(addr  & 0xFF);

for(i=0;i<256;i++)spi1.send_data(buf[i]);

	SPI1_CS_ON;
	
	wait_busy();
}


/*
 * Name								: flash_writen
 * Description				: ---
 * Author							: XiaomaGee.
 *
 * History
 * -------------------
 * Rev								: 0.00
 * Date								: 08/26/2012
 * 
 * create.
 * -------------------
 */
int flash_writen(unsigned long int addr,char *buf, int len)
{
  int cnt = len / 256; //
	int i;
	
  sector_erase(addr);
 	
	enable_flash_write(); 

	 for(i = 0;i< cnt;i++){
		  page_write(addr + i*256,buf + i*256);
	 }
	
	return 0;
}
/*
 * Name								: read_flash_id
 * Description				: ---
 * Author							: XiaomaGee.
 *
 * History
 * -------------------
 * Rev								: 0.00
 * Date								: 08/26/2012
 * 
 * create.
 * -------------------
 */
unsigned long read_flash_id(void) 
{
	unsigned long temp=0;
		
	
	SPI1_CS_OFF;
							
	spi1.send_data(READ_ID);										/* send JEDEC ID command (9Fh) */
	
 	temp = ((temp | spi1.send_data(0)) << 8);		/* receive byte */
	temp = ((temp | spi1.send_data(0)) << 8);	
	temp = (temp |  spi1.send_data(0)); 				/* temp value = 0xBF2541 for sst25vf016b */
											   
														/* disable device */
	SPI1_CS_ON;	
	
	write_status_register(0);
	
	return temp;
}
/*
 * Name								: enable_flash_write
 * Description				: ---
 * Author							: XiaomaGee.
 *
 * History
 * -------------------
 * Rev								: 0.00
 * Date								: 08/26/2012
 * 
 * create.
 * -------------------
 */
void enable_flash_write(void)
{
	SPI1_CS_OFF;

	spi1.send_data(EN_WRITE_FLASH);

	SPI1_CS_ON; 	
}
/*
 * Name								: disable_flash_write
 * Description				: ---
 * Author							: XiaomaGee.
 *
 * History
 * -------------------
 * Rev								: 0.00
 * Date								: 08/26/2012
 * 
 * create.
 * -------------------
 */
void disable_flash_write(void)
{
	SPI1_CS_OFF;

	spi1.send_data(DIS_WRITE_FLASH);

	SPI1_CS_ON;
}
/*
 * Name								: erase_flash
 * Description				: ---
 * Author							: XiaomaGee.
 *
 * History
 * -------------------
 * Rev								: 0.00
 * Date								: 08/26/2012
 * 
 * create.
 * -------------------
 */
void erase_flash(void)
{
	
	enable_flash_write();

	SPI1_CS_OFF;
	spi1.send_data(ERASE);
	SPI1_CS_ON;
	
	wait_busy();	
}

/*
 * Name								: sector_erase
 * Description				: ---
 * Author							: XiaomaGee.
 *
 * History
 * -------------------
 * Rev								: 0.00
 * Date								: 08/26/2012
 * 
 * create.
 * -------------------
 */
void sector_erase(unsigned long int addr)
{
	
	enable_flash_write();
	
	SPI1_CS_OFF;										/* enable device */

	spi1.send_data(SECTOR_ERASE); 					/* read command */
	spi1.send_data(((addr & 0xFFFFFF) >> 16)); 	/* send 3 address bytes */
	spi1.send_data(((addr & 0xFFFF) >> 8));
	spi1.send_data(addr & 0xFF);
	
	SPI1_CS_ON;
	
	wait_busy();	
}

/*
 * Name								: enable_write_status_register
 * Description				: ---
 * Author							: XiaomaGee.
 *
 * History
 * -------------------
 * Rev								: 0.00
 * Date								: 08/26/2012
 * 
 * create.
 * -------------------
 */
void enable_write_status_register(void)
{

	SPI1_CS_OFF;									/* enable device */
	spi1.send_data(EN_WRITE_REGISTER);		/* enable writing to the status register */
	SPI1_CS_ON;										/* disable device */
}
/*
 * Name								: write_status_register
 * Description				: ---
 * Author							: XiaomaGee.
 *
 * History
 * -------------------
 * Rev								: 0.00
 * Date								: 08/26/2012
 * 
 * create.
 * -------------------
 */
void write_status_register(unsigned char byte)
{
	enable_write_status_register();

	SPI1_CS_OFF;				/* enable device */
	spi1.send_data(WRITE_REGISTER);		/* send RDSR command */
	spi1.send_data(byte);		/* receive byte */
	SPI1_CS_ON;				/* disable device */
	
}
/*
 * Name								: read_status_register
 * Description				: ---
 * Author							: XiaomaGee.
 *
 * History
 * -------------------
 * Rev								: 0.00
 * Date								: 08/26/2012
 * 
 * create.
 * -------------------
 */
unsigned char read_status_register(void)
{
	unsigned char byte = 0;

	SPI1_CS_OFF;								/* enable device */

	spi1.send_data(READ_REGISTER);		/* send RDSR command */
	byte = spi1.send_data(0);				/* receive byte */
	
	SPI1_CS_ON;									/* disable device */

	
	return byte;
}
/*
 * Name								: flash_readn
 * Description				: ---
 * Author							: XiaomaGee.
 *
 * History
 * -------------------
 * Rev								: 0.00
 * Date								: 08/26/2012
 * 
 * create.
 * -------------------
 */
int flash_readn(unsigned long addr, char * buf,int byte_num)
{
	unsigned long i = 0;

	SPI1_CS_OFF;										/* enable device */

	spi1.send_data(READ_FLASH_HS); 					/* read command */
	spi1.send_data(((addr & 0xFFFFFF) >> 16)); 	/* send 3 address bytes */
	spi1.send_data(((addr & 0xFFFF) >> 8));
	spi1.send_data(addr & 0xFF);
	
	spi1.send_data(0); //Dummy byte

	for (i = 0; i < byte_num; i++)					
	{
		buf[i] = spi1.send_data(0);	
	}
	
	SPI1_CS_ON;											/* disable device */

	return 0;
}
/*
 * Name								: wait_busy
 * Description				: ---
 * Author							: XiaomaGee.
 *
 * History
 * -------------------
 * Rev								: 0.00
 * Date								: 08/26/2012
 * 
 * create.
 * -------------------
 */
 void wait_busy(void)
{
	while ((read_status_register() & 0x03) == 0x03)		read_status_register();

}
