/*
 * File								: sd.c
 * Description				: This file is iHMI43 sd card driver.
 * Author							: XiaomaGee@Gmail.com
 * Copyright					:
 *
 * History
 *--------------------
 * Rev								: 0.00
 * Date								: 05/9/2012
 *
 * create.
 *--------------------
 */
 
//-----------------Include files-------------------------//
 
#include "..\include\hardware.h"
#include "..\include\spi.h"
#include "..\include\sd.h"

//---------------- Function Prototype ------------------//
static int initialize_sd(void);
static int write_command(unsigned char * cmd);
static int read_block(unsigned char *cmd,unsigned char *buf,unsigned int number);
static int read_cid(unsigned char * buf);
static int read_csd(unsigned char * buf);
static int read_sector(unsigned long int sector_number, unsigned char *buf);
static int write_sector(unsigned long int sector_number,const unsigned char *buf) ;
 

//-----------------Variable-----------------------------//  
SD_T sd={
	.initialize = initialize_sd,
	.read_cid = read_cid,
	.read_csd = read_csd,
	.read_sector=read_sector,
	.write_sector=write_sector
};

//-----------------Function-----------------------------//

/*
 * Name										: initialize_sd
 * Description						: ---
 * Author									: XiaomaGee.
 *
 * History
 * -----------------------
 * Rev										: 0.00
 * Date										: 03/05/2012
 * 
 * create.
 * ----------------------
 */
 static int 
 initialize_sd(void)
{
	unsigned char temp;
	int retry = 0;
	int i;
	unsigned char cmd_list[]={0x40,0x00,0x00,0x00,0x00,0x95};
	
	SPI2_CS_ON;
	
	for(i = 0;i < 15;i ++){ //  > 74 clk
			spi2.send_data(0xff);
	}
	
	do{
		temp  = write_command(cmd_list);
	
		if(retry++ > 1000)return -1;
	}while(temp != 1);

	cmd_list[0] = 0x41; //cmd1
	cmd_list[5] = 0xff; 
	
	retry = 0;
	
	do{
		temp = write_command(cmd_list);
	
		if(retry ++ > 5000)return -1; ///
	}while(temp != 0);

	return 0;
}
/*
 * Name										: write_command
 * Description						: ---
 * Author									: XiaomaGee.
 *
 * History
 * -----------------------
 * Rev										: 0.00
 * Date										: 03/05/2012
 * 
 * create.
 * ----------------------
 */
static int 
write_command(unsigned char * cmd)
{
	unsigned char temp;
	int retry = 0;
	int i;
	
	SPI2_CS_ON;
	spi2.send_data(0xff);
	SPI2_CS_OFF;
	
	for(i = 0;i < 6;i ++){
		spi2.send_data(*cmd ++);
		}
		
		spi2.send_data(0xff);
		
		do{
			temp = spi2.send_data(0xff);
			retry ++;
		}while((temp == 0xff)&&(retry <200));
	
			SPI2_CS_ON;
		return temp;
}
/*
 * Name										: read_block
 * Description						: ---
 * Author									: XiaomaGee.
 *
 * History
 * -----------------------
 * Rev										: 0.00
 * Date										: 03/05/2012
 * 
 * create.
 * ----------------------
 */
static int 
read_block(unsigned char *cmd,unsigned char *buf,unsigned int number)
{
	int i;
	unsigned char temp;
	int retry;
	unsigned char set_blocken[]={0X50,0X00,0X00,0X00,0X00,0XFF};
	
	set_blocken[3] = ((number & 0xff00)>>8);
	set_blocken[4] = (number & 0xff);
	
	retry = 0;
	
	do{
		temp = write_command(set_blocken);
		if(retry ++ > 15000)return -1; //error
	}while(temp != 0);
	
	retry = 0;
	
	do{
		temp = write_command(cmd);
		if(retry ++ > 15000)return -1; //error
	}while(temp != 0);
	
		SPI2_CS_OFF;
	
	retry = 0;
	while(spi2.send_data(0xff) != 0xfe){
		retry ++;
		if(retry > 15000)return -1; //error
	}
	
	
	for(i = 0;i < number ;i ++){
			*buf ++ = spi2.send_data(0xff);
	}
	
	spi2.send_data(0xff);
	spi2.send_data(0xff);
	
	SPI2_CS_ON;
	
	return 0;
	
}
/*
 * Name										: read_cid
 * Description						: ---
 * Author									: XiaomaGee.
 *
 * History
 * -----------------------
 * Rev										: 0.00
 * Date										: 03/05/2012
 * 
 * create.
 * ----------------------
 */
static int 
read_cid(unsigned char * buf)
{
	unsigned char cmd[]={0x4a,0x00,0x00,0x00,0x00,0xff};
	
	return read_block(cmd,buf,16);
 }
 /*
 * Name										: read_csd
 * Description						: ---
 * Author									: XiaomaGee.
 *
 * History
 * -----------------------
 * Rev										: 0.00
 * Date										: 03/05/2012
 * 
 * create.
 * ----------------------
 */
 static int 
 read_csd(unsigned char * buf)
{
	unsigned char cmd[]={0x49,0x00,0x00,0x00,0x00,0xff};
	
	return read_block(cmd,buf,16);
 }
 /*
 * Name										: read_csd
 * Description						: ---
 * Author									: XiaomaGee.
 *
 * History
 * -----------------------
 * Rev										: 0.00
 * Date										: 03/05/2012
 * 
 * create.
 * ----------------------
 */
 static int 
 read_sector(unsigned long int sector_number, unsigned char * buf)
 {
	 unsigned char cmd_list[]={0x51,0x00,0x00,0x00,0x00,0xff};
	 
	 sector_number <<= 9; 
	 cmd_list[1]=((sector_number &0xff000000)>>24);
	 cmd_list[2]=((sector_number &0xff0000)>>16);
	 cmd_list[3]=((sector_number &0xff00)>>8);
	 
	 return read_block(cmd_list,buf,512);
 }
 /*
 * Name										: read_csd
 * Description						: ---
 * Author									: XiaomaGee.
 *
 * History
 * -----------------------
 * Rev										: 0.00
 * Date										: 03/05/2012
 * 
 * create.
 * ----------------------
 */
static int
write_sector(unsigned long int sector_number,const unsigned char *buf) 
 {
	 unsigned char temp;
	 int retry;
	 int i;
	 
	 unsigned char cmd_list[]={0x58,0x00,0x00,0x00,0x00,0xff};
	 
	 sector_number <<= 9;
	 
	 cmd_list[1]= ((sector_number & 0xff000000)>>24);
	 cmd_list[2]= ((sector_number & 0xff0000)>>16);
	 cmd_list[3]= ((sector_number & 0xff00)>>8);
	 
	 retry = 0;
	 
	 do{
		temp = write_command(cmd_list);
		 
		 if(retry++ > 100)return -1; //error
	 }while(temp !=0);
	 
	 for(i = 0;i < 100; i++)spi2.send_data(0xff);
	 
	 SPI2_CS_OFF;
	 
	 spi2.send_data(0xfe);
	 for(i = 0;i < 512;i ++)spi2.send_data(*buf ++);
	 
	 spi2.send_data(0xff);
	 spi2.send_data(0xff);
	 
	 if((spi2.send_data(0xff) &0x1f) != 0X05){
		 SPI2_CS_ON;
		 return -1;
	 }
	 
	 while(spi2.send_data(0xff) !=0xff);
	 
	 SPI2_CS_ON;
	 
	 return 0;
}



