/*
 * File								: main.c
 * Description				: This file is iHMI43 main driver.
 * Author							: XiaomaGee@Gmail.com
 * Copyright					:
 *
 * History
 *--------------------
 * Rev								: 0.00
 * Date								: 05/9/2012
 *
 * create.
 *--------------------
 */

//-----------------Include files-------------------------//

#include "..\include\nvic.h"
#include "..\include\rcc.h"
#include "..\include\usart.h"
#include "..\include\hardware.h"
#include "..\include\i2c.h"
#include "..\include\pwm.h"
#include "..\include\fsmc.h"
#include "..\include\flash.h"
#include "..\include\spi.h"
#include "..\include\evtft.h"
#include "..\include\font.h"
#include "..\fatfs\ff.h"
#include "..\include\systick.h"
#include "..\include\beep.h"
#include "..\include\lm75.h"
#include "..\include\ds1302.h"
#include "..\include\ads7843.h"
#include "..\include\bmp.h"
#include "..\include\touch.h"
#include "..\include\key.h"
#include "..\include\sd.h"

#include "..\include\usb_mass.h"

#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <stdarg.h>
#include <stdio.h>

//---------------- Function Prototype -------------------//
static int virtual_u(void);
static int frame(void);
static int show_time(void);
static int show_temp(void);
static int show_flash(void);
static int process_uart(void);
static int process_touch(void);
static int capture_handle(void);
static int show_message(const char *fmt, ...);

//----------------- Variable -----------------------------/
FATFS	fatfs;
float version = 0.14;
//-----------------Function------------------------------//

/*
 * Name										: main
 * Description						: ---
 * Author									: XiaomaGee.
 *
 * History
 * -----------------------
 * Rev										: 0.00
 * Date										: 03/05/2012
 * 
 * create.
 * ----------------------
 */

int main(void)
{
	rcc.initialize();
	nvic.initialize();
	CAPTURE_INPUT;
	POWER_ON_INPUT;

	usart1.initialize(9600);
	usart2.initialize(9600);
	usart3.initialize(9600);

	fsmc.initialize();
	
	beep.initialize();
	beep.frequency(3000);
	lm75.initialize();

	spi3.initialize();	
	spi2.initialize();
	spi1.initialize();	

	lcd.initialize();
	lcd.cls(COLOR_WINDOW_BACKGROUND);
	
	virtual_u();
	LED_OUTPUT;
	systick.initialize();	
	pwm.open(100);
	beep.frequency(0);
	ads7843.initialize();	

	f_mount(0,&fatfs);

	//ads7843.Touch_Adjust();
	frame();	
	show_flash();
	
	// add touch key
	touch.clear();
	touch.add_node(&touch_key_a);
	touch.add_node(&touch_key_b);
	
	//detect sd card
	show_message("* Initialize TF CARD ... %s",(sd.initialize() == 0)?"OK!":"Failed!");

	systick.second_flag = 1;
	ds1302.initialize();

	while(1){

		process_uart();
		capture_handle();
		process_touch();

		if(systick.second_flag){	
			systick.second_flag = 0;
			show_time();
			show_temp();
		}

	}
}
/*
 * Name										: frame
 * Description								: ---
 * Author									: XiaomaGee.
 *
 * History
 * -----------------------
 * Rev										: 0.00
 * Date										: 03/05/2012
 * 
 * create.
 * ----------------------
 */
static int
frame(void)
{
	BOX_T b;
	STRING_T s;


	//touch key A
 	font._default.double_byte = &simsun16;
	font._default.single_byte = &fixedsys;
	b.attrib = BOX_RAISED;
	b.startx = 330;
	b.starty = 225;
	b.endx= 395;
	b.endy = 265;
	b.color = COLOR_WINDOW_BACKGROUND;

	gui.box(&b);
	
	s.x = 338;
	s.y = 238;
	s.space.line=0;
	s.space.word=0;
	s.color=COLOR_BLACK;
	s.background_color=COLOR_WINDOW_BACKGROUND;	
	s.inverse=NULL; 
	font.printf(&s,"ť A");
	
	//touch key B
	
	b.attrib = BOX_RAISED;
	b.startx = 405;
	b.starty = 225;
	b.endx= 470;
	b.endy = 265;
	b.color = COLOR_WINDOW_BACKGROUND;

	gui.box(&b);

	s.x = 413;
	s.y = 238;
	s.space.line=0;
	s.space.word=0;
	s.color=COLOR_BLACK;
	s.background_color=COLOR_WINDOW_BACKGROUND;	
	s.inverse=NULL; 
	font.printf(&s,"ť B");
	
	font._default.double_byte = &simsun12;
	font._default.single_byte = &fixedsys;	
	
	//rtc
	b.startx = 2;
	b.starty = 20;
	b.endx= 160;
	b.endy = 210;
	b.attrib = BOX_RECESSED;
	b.color = COLOR_BLACK;
  
	gui.box(&b);

	//temp	
	b.startx = 167;
	b.starty = 20;
	b.endx= 320;
	b.endy = 110;
	b.attrib = BOX_FRAME;
	b.color = COLOR_WINDOW_BACKGROUND;
	gui.box(&b);

	s.x = 177;
	s.y = 14;
	s.space.line=0;
	s.space.word=0;
	s.color=COLOR_BLACK;
	s.background_color=COLOR_WINDOW_BACKGROUND;	
	s.inverse=NULL; 
	font.printf(&s," ¶Ȳ ");	

	//Flash
	b.startx = 167;
	b.starty = 120;
	b.endx= 320;
	b.endy = 210;
	b.attrib = BOX_FRAME;
	b.color = COLOR_WINDOW_BACKGROUND;
	gui.box(&b);

	s.x = 177;
	s.y = 114;
	s.space.line=0;
	s.space.word=0;
	s.color=COLOR_BLACK;
	s.background_color=COLOR_WINDOW_BACKGROUND;	
	s.inverse=NULL; 
	font.printf(&s," Flash Ϣ ");	

	//Uart
	b.startx = 13;
	b.starty = 220;
	b.endx= 320;
	b.endy = 270;
	b.attrib = BOX_FRAME;
	b.color = COLOR_WINDOW_BACKGROUND;
	gui.box(&b);

	s.x = 2;
	s.y = 237;
	s.space.line=0;
	s.space.word=0;
	s.color=COLOR_BLACK;
	s.background_color=COLOR_WINDOW_BACKGROUND;	
	s.inverse=NULL; 
	font.printf(&s,"Ϣ");	

	b.startx = 28;
	b.starty = 225;
	b.endx= 315;
	b.endy = 265;
	b.attrib = BOX_RECESSED;
	b.color = COLOR_BLACK;
	gui.box(&b);

	//touch	X
	b.startx = 328;
	b.starty = 20;
	b.endx= 478;
	b.endy = 110;
	b.attrib = BOX_FRAME;
	b.color = COLOR_WINDOW_BACKGROUND;
	gui.box(&b);

	s.x = 338;
	s.y = 14;
	s.space.line=0;
	s.space.word=0;
	s.color=COLOR_BLACK;
	s.background_color=COLOR_WINDOW_BACKGROUND;	
	s.inverse=NULL; 
	font.printf(&s,"  X ");	

	//touch	Y
	b.startx = 328;
	b.starty = 120;
	b.endx= 478;
	b.endy = 210;
	b.attrib = BOX_FRAME;
	b.color = COLOR_WINDOW_BACKGROUND;
	gui.box(&b);

	s.x = 338;
	s.y = 114;
	s.space.line=0;
	s.space.word=0;
	s.color=COLOR_BLACK;
	s.background_color=COLOR_WINDOW_BACKGROUND;	
	s.inverse=NULL; 
	font.printf(&s,"  Y ");	

	//version infomation

	font._default.double_byte = &simsun12;
	font._default.single_byte = &tahoma8;

	s.x = 238;
	s.y = 0;
	s.space.line=0;
	s.space.word=0;
	s.color=COLOR_GRAY;
	s.background_color=COLOR_WINDOW_BACKGROUND;	
	s.inverse=NULL; 
	font.printf(&s,"[iHMI43 demo. ver. %.2f.%s.%s]",version,__DATE__,__TIME__);	


	return 0;
}

/*
 * Name										: show_temp
 * Description						: ---
 * Author									: XiaomaGee.
 *
 * History
 * -----------------------
 * Rev										: 0.00
 * Date										: 03/05/2012
 * 
 * create.
 * ----------------------
 */
static int
show_temp(void)
{
	STRING_T s;

	font._default.double_byte = &simsun16;
	font._default.single_byte = &nix48;

	s.x = 175;
	s.y = 40;
	s.space.line=0;
	s.space.word=0;
	s.color=COLOR_RED;
	s.background_color=COLOR_WINDOW_BACKGROUND;	
	s.inverse=NULL;
	font.printf(&s,"%.1f",lm75.read());	

	return 0;
}
/*
 * Name										: show_flash
 * Description								: ---
 * Author									: XiaomaGee.
 *
 * History
 * -----------------------
 * Rev										: 0.00
 * Date										: 03/05/2012
 * 
 * create.
 * ----------------------
 */

static int
show_flash(void)
{
	STRING_T s;
	unsigned long int id;

	font._default.single_byte = &fixedsys;

	s.x = 174;
	s.y = 135;
	s.space.line = 0;
	s.space.word = 0;
	s.color = COLOR_BLUE;
	s.background_color = COLOR_WINDOW_BACKGROUND;
	s.inverse = NULL;

	font.printf(&s, "ID  : 0x%X \n", id = w25qxx.read_id());
	font.printf(&s, "Manu: %s\n", id >> 16 == 0xef ? "Winbond" : "Unknown");
	font.printf(&s, "Type: %s\n", ((id >> 8) & 0xff) == 0x40 ? "Serial Flash" : "Unknown");
	font.printf(&s, "Size: %.2fMB", pow(2, ((id & 0xff) - 0x14)));

	return 0;
}

/*
 * Name										: virtual_u
 * Description								: ---
 * Author									: XiaomaGee.
 *
 * History
 * -----------------------
 * Rev										: 0.00
 * Date										: 03/05/2012
 * 
 * create.
 * ----------------------
 */
static int
virtual_u(void)       //u
{
	FATFS	fatfs;

	POWER_ON_INPUT;
	if(POWER_ON_DETECT)return 0;

	f_mount(0,&fatfs);
	bmp.show("0:/system/pic/mass.bmp",0,0);
	f_mount(0,NULL);

	pwm.initialize(100);
	beep.frequency(0);
	usb_mass.initialize();

	while(1);
}
/*
 * Name				: tx / ty
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * -----------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * ----------------------
 */	
int tx(int x)
{
	return x + 80;
}

int ty(int y)
{
	return 105 - y;
}

/*
 * Name				: set_time
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * -----------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * ----------------------
 */
static int
show_time(void)
{
	typedef struct {
		int x;
		int y;
	}POINT_T;

#define PI      3.1415926
	extern const unsigned char gImage_clock[];
	unsigned short int *p = (unsigned short int*)gImage_clock;
	float temp;
	POINT_T ph0, ph1, pm0, pm1, ps0, ps1;
	RECT_T r;
	STRING_T s;


	//show_time

	ds1302.get(&rtc);
	systick.second_flag = 0;

	temp = ((rtc.hour + rtc.minute / 60.) / 12.) * 2 * PI;
	ph0.x = 32 * sin(temp);
	ph0.y = 32 * cos(temp);
	ph1.x = 5 * sin(temp);
	ph1.y = 5 * cos(temp);

	temp = ((rtc.minute + rtc.second / 60.) / 60.) * 2 * PI;
	pm0.x = 42 * sin(temp);
	pm0.y = 42 * cos(temp);
	pm1.x = 5 * sin(temp);
	pm1.y = 5 * cos(temp);

	temp = (rtc.second / 60.) * 2 * PI;
	ps0.x = 55 * sin(temp);
	ps0.y = 55 * cos(temp);
	ps1.x = 5 * sin(temp);
	ps1.y = 5 * cos(temp);

	r.x = 12;
	r.y = 37;
	r.width = 135;
	r.height = 135;
	r.fill_flag = FILL_PICTURE;
	r.color = COLOR_RED;
	r.p = p;
	gui.rect(&r);

	gui.line(tx(ph0.x), ty(ph0.y), tx(ph1.x), ty(ph1.y), COLOR_BLUE);
	gui.line(tx(pm0.x), ty(pm0.y), tx(pm1.x), ty(pm1.y), COLOR_GREEN);
	gui.line(tx(ps0.x), ty(ps0.y), tx(ps1.x), ty(ps1.y), COLOR_ORANGE);

	gui.circle(tx(0), ty(0), 4, 0, COLOR_ORANGE);

	font._default.single_byte = &monaco;

	//show date

	s.x = 15;
	s.y = 185;
	s.space.line = 2;
	s.space.word = 0;
	s.color = COLOR_SYSTEM;
	s.background_color = COLOR_BLACK;

	s.inverse = NULL;

	font.printf(&s, "%02d-%02d-%02d %02d:%02d:%02d", rtc.year, rtc.month, rtc.day,rtc.hour, rtc.minute, rtc.second);


	return 0;
}
/*
 * Name										: process_uart
 * Description						: ---
 * Author									: XiaomaGee.
 *
 * History
 * -----------------------
 * Rev										: 0.00
 * Date										: 03/05/2012
 * 
 * create.
 * ----------------------
 */
static int
process_uart(void)
{
	if (usart1.receive_ok_flag) {
		usart1.receive_ok_flag = 0;
		usart1.printf("[UART1]%s",usart1.receive_buffer);		

		show_message("[UART1]%s",usart1.receive_buffer);
	}
	if (usart2.receive_ok_flag) {
		usart2.receive_ok_flag = 0;
		usart2.printf("[UART2]%s",usart2.receive_buffer);			

		show_message("[UART2]%s",usart2.receive_buffer);
	}
	if (usart3.receive_ok_flag) {
		usart3.receive_ok_flag = 0;
		usart3.printf("[UART3]%s",usart3.receive_buffer);
		show_message("[UART3]%s",usart3.receive_buffer);
	}		
	return 0;
}
/*
 * Name										: process_touch
 * Description						: ---
 * Author									: XiaomaGee.
 *
 * History
 * -----------------------
 * Rev										: 0.00
 * Date										: 03/05/2012
 * 
 * create.
 * ----------------------
 */
static int
process_touch(void)
{
	STRING_T s;
	BOX_T b;

	if(systick.msecond_50_flag){
		systick.msecond_50_flag = 0;
		
		font._default.double_byte = &simsun16;
		font._default.single_byte = &nix48;

		s.x = 333;
		s.y = 40;
		s.space.line=0;
		s.space.word=0;
		s.color=COLOR_RED;
		s.background_color=COLOR_WINDOW_BACKGROUND;	
		s.inverse=NULL;
		font.printf(&s,"%03d",Pen_Point.x0);

		s.x = 333;
		s.y = 140;
		s.space.line=0;
		s.space.word=0;
		s.color=COLOR_RED;
		s.background_color=COLOR_WINDOW_BACKGROUND;	
		s.inverse=NULL;
		font.printf(&s,"%03d",Pen_Point.y0);
	}
		
		if(event == EVENT_KEY_A ){
				event = 0;
			
				b.attrib = BOX_RECESSED;
				show_message("ť A !");
				b.startx = 330;
				b.starty = 225;
				b.endx= 395;
				b.endy = 265;
				b.color = COLOR_SYSTEM;

				gui.box(&b);

				s.x = 339;
				s.y = 239;
				s.space.line=0;
				s.space.word=0;
				s.color=COLOR_BLACK;
				s.background_color=COLOR_SYSTEM;	
				s.inverse=NULL; 
				font.printf(&s,"ť A");
			
				b.attrib = BOX_RAISED;
				b.startx = 405;
				b.starty = 225;
				b.endx= 470;
				b.endy = 265;
				b.color = COLOR_WINDOW_BACKGROUND;

				gui.box(&b);

				s.x = 413;
				s.y = 238;
				s.space.line=0;
				s.space.word=0;
				s.color=COLOR_BLACK;
				s.background_color=COLOR_WINDOW_BACKGROUND;	
				s.inverse=NULL; 
				font.printf(&s,"ť B");
			}	
		
		if(event == EVENT_KEY_B ){
				event = 0;
				
				b.attrib = BOX_RECESSED;
				show_message("ť B !");
				b.startx = 405;
				b.starty = 225;
				b.endx= 470;
				b.endy = 265;
				b.color = COLOR_SYSTEM;
				gui.box(&b);	
			
				s.x = 414;
				s.y = 239;
				s.space.line=0;
				s.space.word=0;
				s.color=COLOR_BLACK;
				s.background_color=COLOR_SYSTEM;	
				s.inverse=NULL; 
				font.printf(&s,"ť B");

				b.attrib = BOX_RAISED;
				b.startx = 330;
				b.starty = 225;
				b.endx= 395;
				b.endy = 265;
				b.color = COLOR_WINDOW_BACKGROUND;

				gui.box(&b);
				
				s.x = 338;
				s.y = 238;
				s.space.line=0;
				s.space.word=0;
				s.color=COLOR_BLACK;
				s.background_color=COLOR_WINDOW_BACKGROUND;	
				s.inverse=NULL; 
				font.printf(&s,"ť A");

			}	
 	
	return 0;
}
/*
 * File								: captture_pic
 * Description				: ---
 * Author							: XiaomaGee@Gmail.com
 * Copyright					:
 *
 * History
 *--------------------
 * Rev								: 0.00
 * Date								: 05/9/2012
 *
 * create.
 *--------------------
 */	   
	static int
capture_handle(void)
{
	STRING_T s;
	RECT_T r;

	if(CAPTURE == 0 ){

		bmp.capture();

		lcd.cls(COLOR_BLACK);

		r.x = 70;
		r.y = 90;
		r.width = 180;
		r.height = 35;
		r.fill_flag = 0;
		r.color = COLOR_SYSTEM;
		gui.rect(&r);

		font._default.single_byte = &fixedsys;
		//MESSAGE
		s.x = 80;
		s.y = 100;
		s.space.line = 2;
		s.space.word = 0;
		s.color = COLOR_SYSTEM;
		s.background_color = COLOR_BLACK;
		s.inverse = NULL;
		font.printf(&s, "capture success!!!");

		usb_mass.initialize();

		while (1) ;
	}

	return 0;
}
/*
 * File								: captture_pic
 * Description				: ---
 * Author							: XiaomaGee@Gmail.com
 * Copyright					:
 *
 * History
 *--------------------
 * Rev								: 0.00
 * Date								: 05/9/2012
 *
 * create.
 *--------------------
 */	   
static int 
show_message(const char *fmt, ...)
{
	va_list arg_ptr;
	STRING_T s;
	char buf[200];

	memset(buf, '\0', sizeof(buf));

	va_start(arg_ptr, fmt);
	vsprintf(buf, fmt, arg_ptr);
	va_end(arg_ptr);
	
 	font._default.double_byte = &simsun16;
	font._default.single_byte = &fixedsys;
	s.x = 35;
	s.y = 238;
	s.space.line=0;
	s.space.word=0;
	s.color=COLOR_SYSTEM;
	s.background_color=COLOR_BLACK;	
	s.inverse=NULL;
	font.printf(&s,buf);

	while(s.x < 300)font.printf(&s," "); //ʾ
	
	return 0;
}
